/* smart.c */
/*

Copyright  1999 The Regents of the University of California. 
All Rights Reserved. 

Permission to use, copy, modify, and distribute this software 
and its documentation for educational, research and non-profit 
purposes, without fee, and without a written agreement is hereby 
granted, provided that the above copyright notice, this 
paragraph and the following three paragraphs appear in all copies. 

Permission to incorporate this software into commercial products 
may be obtained by contacting the University of California 
at: Jan Lester,368 Applied Sciences Building, UCSC, Santa Cruz, 
CA 95064 (831) 459-4533. 

This software program and documentation are copyrighted by The 
Regents of the University of California. The software program 
and documentation are supplied "as is", without any accompanying 
services from The Regents. The Regents does not warrant that the
operation of the program will be uninterrupted or error-free. The 
end-user understands that the program was developed for research 
purposes and is advised not to rely exclusively on the program for 
any reason. 

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY 
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL 
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE 
AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN 
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA 
SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
OF CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. 
*/


#include <stdio.h>
#include "smart.h"

#define command_sets smart
#define word83 cmdsets_supported

#if BOOTARCH==PPC
void swizzle(void *buffer, int len);

void swizzle(void *buffer, int len) {
  char *swizbuf, temp;
  swizbuf = (char *) buffer;
  while (len > 0) {
    temp = swizbuf[0];
    swizbuf[0] = swizbuf[1];
    swizbuf[1] = temp;
    swizbuf += 2;
    len -= 2;
  }
}
#else
#define swizzle(buffer,len) /* */
#endif

int ataReadHDIdentity ( int device, struct hd_driveid *buf)
{

	if (ioctl ( device , HDIO_GET_IDENTITY, buf ) != 0)
	{ 
		perror ("ATA GET HD Failed");
		return -1; 
	}

	return 0;
}




int ataVersionInfo ( struct hd_driveid drive)
{

	int i;
	int atavalue = 0; 
	

#ifdef DEBUG

	printf ("Debug: Ata Version:	 %04x\n", drive.word80);

#endif 
	
	if ( (drive.word80 == 0xffff) || (drive.word80 == 0x000) )
	{
		
		return -1;

	}

	for ( i = 1; i < 16; i++ )
	{
		if ( drive.word80 &  ( 0x1 << i) )
		{
			atavalue = i;
		}
	}
	return atavalue;
}




int ataSmartSupport ( struct hd_driveid drive)
{

	if ( drive.command_sets & 0x0001 ){
					
			if ( ataVersionInfo ( drive) > 4)
			{
#ifdef TRUST_WORD_83			   
				if ( (drive.word83 == 0xffff) ||
					 (drive.word83 == 0x0000) )
				{
					return (-1); /* word 83 not implemented on drive per ATA spec */
				}


				if ( drive.word83 & 0x2000 ) {
				
					return (2); /* drive supports S.M.A.R.T. and is enabled */
				}
				else
				{
					return (1); /* drive supports S.M.A.R.T. and is disabled */
				}
#else
			   return (255); /* supported, dunno if enabled */
#endif
			}
			else
			{
			
			return (255); /* drive supports S.M.A.R.T. and stat is not unknown*/

			}
	
			
	}
	else 
	{
		return (-1); /* drive does not support S.M.A.R.T. */
	}

}




int ataReadSmartValues (int device, struct ata_smart_values *data)
{	
    int i;
	unsigned char chksum;

	unsigned char parms[4+512] = { WIN_SMART, 0, SMART_READ_VALUES, 1};
	#define buf (parms+4)

	memset(buf, 0, 512);
	
	if (ioctl ( device , HDIO_DRIVE_CMD,  &parms ) != 0)
	{
		perror ("Smart Values Read failed");
		return -1;
	}
   
	swizzle(buf, 512);

	for ( i = chksum = 0; i < 512; i++ )
		chksum+= buf[i];

	
	if (  chksum != 0)
	{
		perror ("Smart Read Failed, Chksum error");
		return -1;
	}


	memcpy( data, buf, sizeof(buf));

#undef buf

	
	return 0;
}



int ataReadSelfTestLog (int device, struct ata_smart_selftestlog *data)
{	

	unsigned char parms[4+512] = { WIN_SMART, 0x06, SMART_READ_LOG_SECTOR, 1};
	#define buf (parms+4)
	
	if (ioctl ( device , HDIO_DRIVE_CMD,  &parms ) != 0)
	{
		perror ("Smart Error Log Read failed");
		return -1;
	}
	

	swizzle(buf, 512);

	memcpy( data, buf, 512);

#undef buf
	
	return 0;
}



int ataReadErrorLog (int device, struct ata_smart_errorlog *data)
{	

	unsigned char parms[4+512] = { WIN_SMART, 0x01, SMART_READ_LOG_SECTOR, 1};
	#define buf (parms+4)
	
	if (ioctl ( device , HDIO_DRIVE_CMD,  &parms ) != 0)
	{
		perror ("Smart Error Log Read failed");
		return -1;
	}
	

	swizzle(buf, 512);

	memcpy( data, buf, 512);

	#undef buf
	
	return 0;
}



int ataReadSmartThresholds ( int device, struct ata_smart_thresholds *data)
{	
	unsigned char parms[4+512] = { WIN_SMART, 1, SMART_READ_THRESHOLDS, 1};
	#define buf (parms+4)
	
	
	if (ioctl ( device , HDIO_DRIVE_CMD,  &parms ) != 0)
	{
		perror ("Smart Thresholds Read failed");
		return -1;
	}
	swizzle(buf, 512);

	memcpy( data, buf, sizeof(buf));

#undef buf

	return 0;
}

int ataSetSmartThresholds ( int device, struct ata_smart_thresholds *data)
{	
	unsigned char parms[4+512] = { WIN_SMART, 1, 0xD7, 1};
	#define buf (parms+4)
	
	
	if (ioctl ( device , HDIO_DRIVE_CMD,  &parms ) != 0)
	{
		perror ("Smart Thresholds Read failed");
		return -1;
	}
	memcpy( data, buf, sizeof(buf));

#undef buf

	return 0;
}


int ataEnableSmart (int device )
{	

	
	unsigned char parms[4] = { WIN_SMART, 1, SMART_ENABLE, 0};
	
	if (ioctl ( device , HDIO_DRIVE_CMD,  &parms, NULL ) != 0)
	{
		perror ("Smart Enable failed");
		return -1;
	}
	
	return 0;
}


int ataDisableSmart (int device )
{	

	
	unsigned char parms[4] = { WIN_SMART, 1, SMART_DISABLE, 0};
	
	if (ioctl ( device , HDIO_DRIVE_CMD,  &parms ) != 0)
	{
		perror ("Smart Disable failed");
		return -1;
	}

	return 0;
}


int ataEnableAutoOffline (int device )
{	

	/* timer hard coded to 4 hours */
	unsigned char parms[4] = { WIN_SMART, 248, SMART_AUTO_OFFLINE, 0};
	
	if (ioctl ( device , HDIO_DRIVE_CMD,  &parms ) != 0)
	{
		perror ("Smart Enable Automatic Offline failed");
		return -1;
	}

	return 0;
}


int ataDisableAutoOffline (int device )
{	

	
	unsigned char parms[4] = { WIN_SMART, 0, SMART_AUTO_OFFLINE, 0};
	
	if (ioctl ( device , HDIO_DRIVE_CMD,  &parms ) != 0)
	{
		perror ("Smart Disable Automatic Offline failed");
		return -1;
	}

	return 0;
}


int ataSmartStatus (int device )
{	

	
	unsigned char parms[4] = { WIN_SMART, 1, SMART_STATUS, 0};
	

	if (ioctl ( device , HDIO_DRIVE_CMD,  &parms) != 0)
	{
#ifdef DEBUG2
		perror ("Smart Status failed");
#endif
		return -1;
	}

	return 0;
}



int ataSmartTest (int device, int testtype)
{	

		unsigned char parms[4] = { WIN_SMART, testtype, SMART_IMMEDIATE_OFFLINE	, 0};
	
	if (ioctl ( device , HDIO_DRIVE_CMD, &parms) != 0)
	{
		perror ("Smart Offline failed");
		return -1;
	
	}
	printf("Completed Off-line command\n");
	
	return 0;
}




int ataSmartOfflineTest (int device)
{	
	return ataSmartTest( device, OFFLINE_FULL_SCAN  );
}



int ataSmartShortSelfTest (int device)
{	
	return ataSmartTest( device, SHORT_SELF_TEST  );
}


int ataSmartExtendSelfTest (int device)
{	
	return ataSmartTest( device, EXTEND_SELF_TEST  );
}


int ataSmartShortCapSelfTest (int device)
{	
	return ataSmartTest( device, SHORT_CAPTIVE_SELF_TEST );
}


int ataSmartExtendCapSelfTest (int device)
{
	return ataSmartTest( device, EXTEND_CAPTIVE_SELF_TEST );
}


int ataSmartSelfTestAbort (int device)
{
	return ataSmartTest( device, 127 );
}

/* Test Time Functions */


int isOfflineTestTime ( struct ata_smart_values data)
{
	return (int) data.total_time_to_complete_off_line;
}



int isShortSelfTestTime ( struct ata_smart_values data)
{
	return (int) data.short_test_completion_time;
}



int isExtendedSelfTestTime ( struct ata_smart_values data)
{
	return (int) data.extend_test_completion_time;
}



int isSmartErrorLogCapable ( struct ata_smart_values data)
{
	return data.errorlog_capability & 0x01;
}



int isSupportExecuteOfflineImmediate ( struct ata_smart_values data)
{
	return data.offline_data_collection_capability & 0x01;
}



int isSupportAutomaticTimer ( struct ata_smart_values data)
{
	return data.offline_data_collection_capability & 0x02;
}



int isSupportOfflineAbort ( struct ata_smart_values data)
{
	return data.offline_data_collection_capability & 0x04;
}



int isSupportOfflineSurfaceScan ( struct ata_smart_values data)
{
	return data.offline_data_collection_capability & 0x08;
}




int isSupportSelfTest (struct ata_smart_values data)
{
	return data.offline_data_collection_capability & 0x10;
}

